;;; aldor-mode.el --- aldor fontification

;; modified by Jeremy (Ya-Yu) Hsieh <hsiehy@unbc.ca>

;; This file is part of Aldor mode for Emacs.

;;; Code:


;; requires
(require 'cl)
(add-to-list 'load-path (file-name-directory load-file-name))
(require 'aldor-region-in)
(require 'aldor-internal)
(require 'aldor-parser)
(require 'aldor-tools)

;; Syntax tables.

;;;###autoload
(defvar aldor-syntax-table
  ;; Syntax table used in aldor-mode buffers.
  ;; Populate the syntax TABLE
  ;; Note that  _ (UNDERSCORE) is the "escape" character in ALDOR!
  (let ((table (make-syntax-table)))
    (let ((i 0))
      (while (< i ?0)
	(modify-syntax-entry i "_   " table)
	(setq i (1+ i)))
      (setq i (1+ ?9))
      (while (< i ?A)
	(modify-syntax-entry i "_   " table)
	(setq i (1+ i)))
      (setq i (1+ ?Z))
      (while (< i ?a)
	(modify-syntax-entry i "_   " table)
	(setq i (1+ i)))
      (setq i (1+ ?z))
      (while (< i 128)
	(modify-syntax-entry i "_   " table)
	(setq i (1+ i)))

      (modify-syntax-entry ?_  "\\"     table)
      (modify-syntax-entry ?+  ". 12"     table)
      (modify-syntax-entry ?-  ". 12"     table)
      (modify-syntax-entry ?=  "."     table)
      (modify-syntax-entry ?%  "."     table)
      (modify-syntax-entry ?<  "."     table)
      (modify-syntax-entry ?#  "."     table)
      (modify-syntax-entry ?>  "."     table)
      (modify-syntax-entry ?&  "."     table)
      (modify-syntax-entry ?|  "."     table)
      (modify-syntax-entry ?*  "."   table)
      (modify-syntax-entry ?  "    " table)
      (modify-syntax-entry ?\t "    " table)
      (modify-syntax-entry ?\n "> b"  table)
      (modify-syntax-entry ?\^m "> b" table)
      (modify-syntax-entry ?\015 "> b" table)
      (modify-syntax-entry ?\( "()  " table)
      (modify-syntax-entry ?\) ")(  " table)
      (modify-syntax-entry ?\[ "(]  " table)
      (modify-syntax-entry ?\] ")[  " table)
      (modify-syntax-entry ?\{ "(}  " table)
      (modify-syntax-entry ?\} "){  " table))
    table))

;;; Variables that are not customizable.

(defvar aldor-mode-shared-map
  (let ((map (make-sparse-keymap)))
    (define-key map "\t" 'aldor-indent-line)
    ;; Here is where my aldor-parse tree kick in..... (unfinish)

    (define-key map "\177" 'backward-delete-char-untabify)
    ;; This gets in the way when viewing a ALDOR file in view-mode.  As
    ;; long as [backspace] is mapped into DEL via the
    ;; function-key-map, this should remain disabled!!
    ;;;(define-key map [backspace] 'backward-delete-char-untabify)
    map)
  "Keymap for commands shared by all sorts of Aldor modes.")


(defvar aldor-mode-map ()
  "Keymap for Aldor mode.
All commands in `aldor-mode-shared-map' are inherited by this map.")

(if aldor-mode-map
    ()
  (let ((map (make-sparse-keymap "Aldor")))
    (setq aldor-mode-map (make-sparse-keymap))
    (set-keymap-parent aldor-mode-map aldor-mode-shared-map)
    (define-key aldor-mode-map "\e\c" 'aldor-comment-region)
    (define-key aldor-mode-map "\e\r" 'aldor-print-tree)
    (define-key aldor-mode-map "\e\p" 'aldor-parse-this-buffer)
    (define-key aldor-mode-map [menu-bar] (make-sparse-keymap))
    (define-key aldor-mode-map [menu-bar aldor]
      (cons "Aldor" map))

    (define-key map [separator-eval] '("--"))
    (define-key map [aldor-print-tree] '("Print Tree" . aldor-print-tree))

    (define-key map [separator-format] '("--"))
    (define-key map [aldor-comment-region] '("Comment Out Region" . aldor-comment-region))
    (define-key map [aldor-parse-tree] '("Parse this buffer" . aldor-parse-this-buffer))
    (put 'eval-region 'menu-enable 'mark-active)
    (put 'comment-region 'menu-enable 'mark-active)
    (put 'indent-region 'menu-enable 'mark-active)))


(defun add-hook-to-region (beg end)
  (interactive "r")
  (put-text-property
   beg end 'insert-behind-hooks (list 'aldor-scan-region)))

(defvar aldor-mode nil
  "Non-nil means aldor Lock mode is active.")
(make-variable-buffer-local 'aldor-mode)

(defvar aldor-mode-functions nil
  "Functions to do the actual fontification.
They are called with two arguments: the START and END of the region to fontify.")
(make-variable-buffer-local 'aldor-mode-functions)

;;; ALDOR mode

(define-derived-mode aldor-mode nil "Aldor"
  "Major mode for editing Aldor code to run in Emacs."

  (aldor-scan buffer-file-name)
  (add-hook-to-region (point-min) (point-max))
  (setq imenu-case-fold-search t))

  
(provide 'aldor-mode)

;;; aldor-mode.el ends here
