;;; aldor-parser.el --- The heart of external version of Aldor Parser

;; Author: Jeremy (Ya-Yu) Hsieh <hsiehy@unbc.ca>
;; Maintainer: None

;; This file contains the code for Aldor Parser


;;; Commentary:

;; This file contains the code and implementation for the
;; Aldor parser.  



;; "aldor-parser-done" variable is declare as global variable
;; the propose of this variable is to determine if the
;; scanner had finish scanning the whole file AND if all
;; of the information in middle.txt had been process.

(defvar aldor-parser-done nil)


;; "aldor-process-done-sentinel" uses as the sentinel to determine
;; if the state of process had been changed.  If the state
;; had been changed, and had been changed as any of the following
;; state - exit, signal, close, or nil; then set the variable
;; "aldor-scanner-done" to t (true).  For more detail about sentinel
;; please refer to my documentation/report/Thesis or look up in
;; Emacs lisp manual (C-h i -> Emacs -> Emacs Lisp -> Sentinels) 

(defun aldor-process-done-sentinel (process string)
  (when (memq (process-status process) '(exit signal closed nil))
    (setq aldor-parser-done t)))


(defun aldor-parser (aldor-file-name)
  (interactive "fFile to parse? ")
  (let
      ((exec-path (cons default-directory exec-path))
       (parser-buffer (get-buffer-create "aldor_parse_middle.el"))
       (parse-proc nil))
    (with-current-buffer parser-buffer
      (erase-buffer))
    (setq aldor-parser-done nil)
    (setq parse-proc (start-process "aldor-parser" parser-buffer 
				    "aldor_parser" aldor-file-name))
    (set-process-sentinel parse-proc 'aldor-process-done-sentinel)
    (while (sit-for 0.1))
    (with-current-buffer (find-file-noselect aldor-file-name)
      (goto-char (point-min))
      (if (eq "Syntax" (read (get-buffer "aldor_parse_middle.el")))
	  ((new-frame)
	   (switch-to-buffer-other-frame tree-buffer))
	(;;    (remove-overlay-prop 'aldor-syntax)
	 (eval-buffer parser-buffer)))
      )
    )
  )

(defun aldor-parse-this-buffer ()
  (interactive)
  (let
      ((exec-path (cons default-directory exec-path))
       (parser-buffer (get-buffer-create "aldor_parse_middle.el"))
       (parse-proc nil))
    (with-current-buffer parser-buffer
      (erase-buffer))
    (setq aldor-parser-done nil)
    (setq parse-proc (start-process "aldor-parser" parser-buffer 
				    "aldor_parser" buffer-file-name))
    (set-process-sentinel parse-proc 'aldor-process-done-sentinel)
    (while (sit-for 0.1))
    ;;    (aldor-remove-overlay-prop 'aldor-syntax)
    (eval-buffer parser-buffer)))



(defun aldor-remove-overlay-prop (prop)
  (save-excursion
    (while (not (eobp))
      (goto-char (min (next-overlay-change (point))
		      (next-single-property-change (point) 'aldor-syntax))))
    (let ((the-overlays (overlays-at point)))
      (delete-overlay the-overlays))))


(defun query-syntax (point)
  (interactive "d")
  (let* ((the-overlays (overlays-at point))
	 (the-syntax (mapcar (lambda (ov)
			       (overlay-get ov 'aldor-syntax))
			     the-overlays)))
    (message "The syntax at %d is %S" point the-syntax)))


;; "aldor-print-tree" is a funtion which will print the tree
;; which user request.  This function takes one argument, which
;; is the tree source string ("aldor-tree-name").  It will then
;; run the process "tree".  Meanwhile, this function will create
;; a new frame and put the result tree to the other frame.
;; For more information please refer to my thesis

(defun aldor-print-tree (aldor-tree-name)
  (interactive "fFile for the tree? ")
  (let
      ((exec-path (cons default-directory exec-path))
       (tree-buffer (get-buffer-create "aldor_tree.txt"))
       (tree-proc nil))
    (with-current-buffer tree-buffer
      (erase-buffer))
    (setq tree-proc (start-process "aldor-tree" tree-buffer 
				   "aldor_tree" aldor-tree-name))
    (switch-to-buffer-other-frame tree-buffer)))

(provide 'aldor-parser)